#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
#include <string>

BLECharacteristic *pCharacteristic;
bool deviceConnected = false;
float txValue = 0;
const int readPin = 25;  // Use GPIO number. See ESP32 board pinouts
const int LED = 2;       // Could be different depending on the dev board. I used the DOIT ESP32 dev board.

bool String_Send;
String Send_Data = "";

std::string rxValue;  // Could also make this a global var to access it in loop()

#define SERVICE_UUID "6E400001-B5A3-F393-E0A9-E50E24DCCA9E"  // UART service UUID
#define CHARACTERISTIC_UUID_RX "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC_UUID_TX "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"

class MyServerCallbacks : public BLEServerCallbacks {
  void onConnect(BLEServer *pServer) {
    deviceConnected = true;

    Send_Data = "Connected";
    String_Send = true;

    Serial.println("Connected");

    // pCharacteristic->setValue("Connected");
    // pCharacteristic->notify();  // Send the value to the app!
  };

  void onDisconnect(BLEServer *pServer) {
    deviceConnected = false;

    Serial.println("Disconnected");


    delay(1000);
    // pService->stop();
    delay(1000);

    // pService->start();

    // Start advertising
    pServer->getAdvertising()->start();
    Serial.println("Waiting a client connection to notify...");
  }
};

class MyCallbacks : public BLECharacteristicCallbacks {
  void onWrite(BLECharacteristic *pCharacteristic) {
    // std::string rxValue = pCharacteristic->getValue();

    // String rxValueString = pCharacteristic->getValue();
    // std::string rxValue = rxValueString.c_str();

    String rxValueString = pCharacteristic->getValue();
    rxValue = rxValueString.c_str();

    if (rxValue.length() > 0) {
      Serial.println("*********");
      Serial.print("Received Value: ");

      for (int i = 0; i < rxValue.length(); i++) {
        Serial.print(rxValue[i]);
      }

      Serial.println();

      // Do stuff based on the command received from the app
      if (rxValue.find("RELAY_ON") != -1) {
        Serial.println("Turning ON!");
        Send_Data = "RELAY_ON";
        String_Send = true;

        digitalWrite(LED, HIGH);
      } else if (rxValue.find("RELAY_OFF") != -1) {
        Serial.println("Turning OFF!");
        Send_Data = "RELAY_OFF";
        String_Send = true;
        digitalWrite(LED, LOW);
      }

      if (rxValue.find("RELAY_2_ON") != -1) {
        Serial.println("Turning ON!");
        Send_Data = "RELAY_2_ON";
        String_Send = true;

        digitalWrite(LED, HIGH);
      } else if (rxValue.find("RELAY_2_OFF") != -1) {
        Serial.println("Turning OFF!");
        Send_Data = "RELAY_2_OFF";
        String_Send = true;
        digitalWrite(LED, LOW);
      }

      if (rxValue.find("RELAY_3_ON") != -1) {
        Serial.println("Turning ON!");
        Send_Data = "RELAY_3_ON";
        String_Send = true;

        digitalWrite(LED, HIGH);
      } else if (rxValue.find("RELAY_3_OFF") != -1) {
        Serial.println("Turning OFF!");
        Send_Data = "RELAY_3_OFF";
        String_Send = true;
        digitalWrite(LED, LOW);
      }

      Serial.println();
      Serial.println("*********");
    }
  }
};

void setup() {
  Serial.begin(115200);

  pinMode(LED, OUTPUT);

  // Create the BLE Device
  BLEDevice::init("ESP32 UART Test");  // Give it a name

  // Create the BLE Server
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);

  // Create a BLE Characteristic
  pCharacteristic = pService->createCharacteristic(
    CHARACTERISTIC_UUID_TX,
    BLECharacteristic::PROPERTY_NOTIFY);

  pCharacteristic->addDescriptor(new BLE2902());

  BLECharacteristic *pCharacteristic = pService->createCharacteristic(
    CHARACTERISTIC_UUID_RX,
    BLECharacteristic::PROPERTY_WRITE);

  pCharacteristic->setCallbacks(new MyCallbacks());

  // Start the service
  pService->start();

  // Start advertising
  pServer->getAdvertising()->start();
  Serial.println("Waiting a client connection to notify...");
}

void loop() {
  if (String_Send) {
    String_Send = false;
    Serial.println("!!!!!!!!!!");
    for (int i = 0; i < 2; i++) {
      pCharacteristic->setValue(Send_Data);  // Sending a test message
      pCharacteristic->notify();               // Send the value to the app!
      delay(1000);
    }
  }
}